//
//  Player.m
//  SuperKoalio
//
//  Created by Jake Gundersen on 12/27/13.
//  Copyright (c) 2013 Razeware, LLC. All rights reserved.
//

#import "Player.h"
#import "SKTUtils.h"

@implementation Player
//1
- (instancetype)initWithImageNamed:(NSString *)name
{
  if (self == [super initWithImageNamed:name]) {
    self.velocity = CGPointMake(0.0, 0.0);
    self.color = [SKColor blackColor];
  }
  return self;
}

- (void)update:(NSTimeInterval)delta
{
  CGPoint gravity = CGPointMake(0.0, -450.0);
  CGPoint gravityStep = CGPointMultiplyScalar(gravity, delta);
  
  float dashScale = 1.0;
  if (self.hud.shouldDash) {
    dashScale = 2.5;
    self.colorBlendFactor = 0.4;
    self.size = CGSizeMake(self.texture.size.width * 1.2, self.texture.size.height * 0.8);
  } else {
    self.colorBlendFactor = 0.0;
    self.size = CGSizeMake(self.texture.size.width * 1.0, self.texture.size.height * 1.0);
  }
  CGPoint forwardMove = CGPointMake(800.0, 0.0);
  CGPoint forwardMoveStep = CGPointMultiplyScalar(forwardMove, delta);

  self.velocity = CGPointAdd(self.velocity, gravityStep);

  self.velocity = CGPointMake(self.velocity.x * 0.9, self.velocity.y);

  CGPoint jumpForce = CGPointMake(0.0, 310.0);
  float jumpCutoff = 150.0;

  if (self.hud.aPushed && self.onGround) {
    self.velocity = CGPointAdd(self.velocity, jumpForce);
    [self runAction:[SKAction playSoundFileNamed:@"jump.wav" waitForCompletion:NO]];
  } else if (!self.hud.aPushed && self.velocity.y > jumpCutoff) {
    self.velocity = CGPointMake(self.velocity.x, jumpCutoff);
  }

  forwardMoveStep = CGPointMultiplyScalar(forwardMoveStep, dashScale * self.hud.xJoystickVelocity);
  self.velocity = CGPointAdd(self.velocity, forwardMoveStep);
  
  CGPoint minMovement = CGPointMake(-120.0 * dashScale, -450);
  CGPoint maxMovement = CGPointMake(120.0 * dashScale, 250.0);
  self.velocity = CGPointMake(Clamp(self.velocity.x, minMovement.x, maxMovement.x), Clamp(self.velocity.y, minMovement.y, maxMovement.y));
  
  CGPoint velocityStep = CGPointMultiplyScalar(self.velocity, delta);
  
  self.desiredPosition = CGPointAdd(self.position, velocityStep);
}

- (CGRect)collisionBoundingBox
{
  CGRect boundingBox = CGRectInset(self.frame, 2, 0);
  CGPoint diff = CGPointSubtract(self.desiredPosition, self.position);
  return CGRectOffset(boundingBox, diff.x, diff.y);
}


@end
